indexing

	description: "Class that can produce output for a certain resource (url). Like a CIG program, but compiled into the HTTP server."

	usage: "Preferably, a servlet should not send something straight to the client. It's better to build the header and body in a STRING and send it as one block."

	author: "Berend de Boer"
	date: "$Date: 2003/11/29 $"
	revision: "$Revision: #3 $"


class

	EPX_HTTP_SERVLET


inherit

	EPX_XHTML_WRITER

	EPX_HTTP_METHODS
		export
			{NONE} all
		end

	EPX_MIME_FIELD_NAMES
		export
			{NONE} all
		end


feature -- Access

	allow_cached_content: BOOLEAN
			-- Can content in `as_string' survive an activate/deactivate sequence?

	allowed_methods: STRING is
			-- What methods does the resource captured by this servlet
			-- allow?
		once
			Result := ""
		ensure
			allowed_methods_not_void: Result /= Void
		end

	is_activated: BOOLEAN is
			-- Has `activate' been called?
		do
			Result := server /= Void
		end

	resource_location: STRING
			-- Location of resource.
			-- Set by `activate'.


feature -- Start/stop

	shutdown is
			-- Send to every servlet when server goes down.
		do
			-- do nothing
		end


feature {EPX_HTTP_SERVER} -- Execution

	activate (a_server: EPX_HTTP_SERVER; a_http_method, a_resource: STRING) is
			-- Called before the servlet is asked to produce output.
			-- Servlet is asked to act upon `a_resource' with method
			-- `a_http_method'.
		require
			not_activated: not is_activated
			no_tag_started: not allow_cached_content implies not is_tag_started
			empty_body: not allow_cached_content implies as_string.is_empty
			server_not_void: a_server /= Void
			client_connected: a_server.has_connected_client
			http_method_not_empty: a_http_method /= Void and then not a_http_method.is_empty
			a_resource_not_empty: a_resource /= Void and then not a_resource.is_empty
		do
			server := a_server
			resource_location := a_resource
		end

	deactivate is
			-- Called after servlet has done its work.
		require
			activated: is_activated
		do
			server := Void
			if not allow_cached_content then
				clear
			end
			resource_location := Void
		ensure
			not_activated: not is_activated
			no_tag_started: not allow_cached_content implies not is_tag_started
			empty_body: not allow_cached_content implies as_string.is_empty
		end

	delete_body is
			-- Write response body to DELETE request.
		require
			activated: is_activated
			body_not_send: not server.is_body_send
		do
			-- not implemented
		end

	delete_header is
			-- Write response header to DELETE request. You have to
			-- write everything, including the response code.
		require
			activated: is_activated
		do
			server.report_method_not_allowed (http_method_DELETE, allowed_methods)
		end

	get_body is
			-- Write response body to GET request.
		require
			activated: is_activated
			body_not_send: not server.is_body_send
		do
			-- not implemented
		end

	get_header is
			-- Write response header to GET request. You have to
			-- write everything, including the response code.
		require
			activated: is_activated
		do
			server.report_method_not_allowed (http_method_GET, allowed_methods)
		ensure
			no_body_separator_added:
				not server.is_header_send implies
					not server.response_header_ends_with_body_separator
		end

	head_header is
			-- Write response header to HEAD request. You have to
			-- write everything, including the response code.
		require
			activated: is_activated
		do
			server.report_method_not_allowed (http_method_HEAD, allowed_methods)
		end

	post_body is
			-- Write response body to POST request.
		require
			activated: is_activated
			body_not_send: not server.is_body_send
		do
			-- not implemented
		end

	post_header is
			-- Write response header to POST request. You have to
			-- write everything, including the response code.
		require
			activated: is_activated
		do
			server.report_method_not_allowed (http_method_POST, allowed_methods)
		end

	put_body is
			-- Write response body to PUT request.
		require
			activated: is_activated
			body_not_send: not server.is_body_send
		do
			-- not implemented
		end

	put_header is
			-- Write response header to PUT request. You have to
			-- write everything, including the response code.
		require
			activated: is_activated
		do
			server.report_method_not_allowed (http_method_PUT, allowed_methods)
		end

	server: EPX_HTTP_SERVER
			-- Server to which this servlet is currently connected.


feature {NONE} -- Header

	add_content_length is
			-- Add a Content-Length header based on the currently
			-- generated XML in `as_uc_string'.
			-- Assume output is written as UTF-8
		require
			header_not_send: not server.is_header_send
		do
			server.add_field (field_name_content_length, as_uc_string.byte_count.out)
		end

	write_default_header is
			-- If you implement a method, a call to this routine will
			-- suffice in most cases.
			-- The header will not the separator and is not send to the client.
		require
			activated: is_activated
			response_header_empty: server.response_header.is_empty
		do
			server.write_response_code (200)
			server.write_mime_header (Void)
		ensure
			header_has_no_body_separator: not server.response_header_ends_with_body_separator
		end


feature -- Values

	adjusted_item (a_key: STRING): STRING is
			-- Value defined for `a_key' if it exists, else the empty string.
			-- Item is left and right trimmed for spaces, tabs, LF's and CR's.
		require
			key_not_empty: a_key /= Void and then not a_key.is_empty
		local
			i: INTEGER
		do
			Result := item (a_key)
			if not Result.is_empty then
				-- right adjust
				Result := clone (Result)
				from
					i := Result.count
				variant
					i
				until
					i < 1 or else
					not blanks.has (Result.item (i))
				loop
					i := i - 1
				end
				Result.keep_head (i)
				-- left adjust
				if not Result.is_empty then
					from
						i := 1
					variant
						Result.count - i
					until
						i > Result.count or else
						not blanks.has (Result.item (i))
					loop
						i := i + 1
					end
					Result.keep_tail ((Result.count - i) + 1)
				end
			end
		ensure
			item_not_void: Result /= Void
			value_returned:
				server.request_form_fields.has (a_key) implies
					server.request_form_fields.item (a_key).value.has_substring (Result)
			empty_string_returned:
				not server.request_form_fields.has (a_key) implies
					Result.is_empty
			does_not_start_with_a_blank: not Result.is_empty implies not blanks.has (Result.item (1))
			does_not_end_with_a_blank: not Result.is_empty implies not blanks.has (Result.item (Result.count))
		end

	item (a_key: STRING): STRING is
			-- Value defined for `a_key' if it exists, else the empty string.
		require
			key_not_empty: a_key /= Void and then not a_key.is_empty
		do
			if server.request_form_fields.has (a_key) then
				Result := server.request_form_fields.item (a_key).value
			else
				Result := once_empty_string
			end
		ensure
			item_not_void: Result /= Void
			value_returned:
				server.request_form_fields.has (a_key) implies
					STRING_.same_string (server.request_form_fields.item (a_key).value, Result)
			empty_string_returned:
				not server.request_form_fields.has (a_key) implies
					Result.is_empty
		end


feature {NONE} -- Once strings

	once_empty_string: STRING is ""

	blanks: STRING is " %T%N%R"


invariant

	activate_implies_server: is_activated = (server /= Void)
	activate_implies_resource_location:
		is_activated implies
			resource_location /= Void and then not resource_location.is_empty

end
