indexing

	description: "Lexical analyzer input buffer which can detect separation between header and body of request, and not read more than necessary to prevent blocking."

	author: "Berend de Boer"
	date: "$Date: 2003/05/15 $"
	revision: "$Revision: #2 $"

class

	EPX_MIME_BUFFER


inherit

	EPX_STREAM_BUFFER
		redefine
			fill,
			wipe_out
		end


creation

	make,
	make_with_size


feature -- Access

	end_of_file_on_end_of_header: BOOLEAN
			-- Don't try to read more than necessary.

	content_length: INTEGER
			-- Read `content_length' bytes, and after that return end-of-file.

	set_end_of_file_on_end_of_header (enable: BOOLEAN) is
			-- Set `end_of_file_on_end_of_header'.
		do
			end_of_file_on_end_of_header := enable
			content_length := 0
			consecutive_end_of_lines := 0
			end_of_file := False
			filled := True
		ensure
			end_of_file_on_end_of_header_set: end_of_file_on_end_of_header = enable
			not_eof: not end_of_file
			try_to_refill: filled
		end

	set_end_of_file_on_content_length (a_content_length, current_position: INTEGER) is
			-- Set `end_of_file_on_content_length'.
			-- Buffer may already contain characters that have to be
			-- returned first, so `current_position' indicates the
			-- position in the buffer that has been read up to now.
		require
			content_length_not_negative: a_content_length >= 0
			current_position_positive: current_position >= 1
			current_position_valid: current_position <= count + 1
		local
			remaining_characters_in_buffer: INTEGER
		do
			content_length := a_content_length
			remaining_characters_in_buffer := (count - current_position) + 1
			if remaining_characters_in_buffer > content_length then
				fix_garbage_after_content_length (remaining_characters_in_buffer)
			else
				bytes_read := remaining_characters_in_buffer
			end
			-- We have to set `end_of_file' in case we already have read
			-- everything there is.
			-- Greater than is probably not possible, but just to be sure.
			end_of_file := bytes_read = content_length
			end_of_file_on_end_of_header := False
			filled := True
		ensure
			end_of_file_on_end_of_header_set: a_content_length = content_length
			try_to_refill: filled
		end


feature -- Element change

	fill is
		local
			c: CHARACTER
			i: INTEGER
		do
			precursor
			if not end_of_file then
				if end_of_file_on_end_of_header then
					from
						i := 0
					until
						consecutive_end_of_lines = 2 or else
						i = count
					loop
						c := content.peek_character (i)
						inspect c
						when '%R' then
							--ignore
						when '%N' then
							consecutive_end_of_lines := consecutive_end_of_lines + 1
						else
							consecutive_end_of_lines := 0
						end
						i := i + 1
					end
					end_of_file := consecutive_end_of_lines = 2
				elseif content_length > 0 then
					-- Assuming `file'.`last_read' is valid, is a tricky one.
					-- Move this stuff to EPX_STREAM_BUFFER?
					if bytes_read + file.last_read > content_length then
						fix_garbage_after_content_length (bytes_read + file.last_read)
					else
						bytes_read := bytes_read + file.last_read
					end
					-- Also assuming we cannot overshoot `content_length'
					-- is a very dangerous assumption
					end_of_file := bytes_read >= content_length
				end
			end
		end

	wipe_out is
		do
			precursor
			bytes_read := 0
		end


feature {NONE} -- Implementation

	bytes_read: INTEGER
			-- How many bytes have been read from` file' so far?

	consecutive_end_of_lines: INTEGER
			-- How many consecutive %R%N (or just %N)'s have been seen in
			-- the input so far?

	fix_garbage_after_content_length (new_bytes_read: INTEGER) is
			-- There is some shit out there, guess the
			-- manufacturer, that sends more than Content-Length.
			-- That would cause `bytes_read' to become larger as
			-- `content_length'. If that is going to be the case, call
			-- this routine.
			-- Note that we assume that Content-Length is correct, and
			-- that any more characters are garbage basically.
		require
			new_bytes_read_larger_as_content_length: new_bytes_read > content_length
		do
			count := count - (new_bytes_read - content_length)
			content.put (End_of_buffer_character, count + 1)
			content.put (End_of_buffer_character, count + 2)
			-- And just say we haven't read more than allowed.
			bytes_read := content_length
		end


invariant

	bytes_read_not_negative: bytes_read >= 0
	consecutive_end_of_lines_not_negative: consecutive_end_of_lines >= 0
	stay_within_the_limit: content_length > 0 implies bytes_read <= content_length
	only_one_tricked_eof_active: not (end_of_file_on_end_of_header and content_length > 0)

end
